#!/usr/bin/env rexx
/* Called by FXMLLoader on the "FX Application Thread" */
/* Invoked via JavaFX using the Java scripting framework, hence RexxScriptEngine executes the prolog code */

d1 = .dateTime~new     -- Save invocation date and time
parse source s . fn    -- Get fully qualified file name as supplied by the RexxScriptEngine

bDebug = .true         -- Change to .false to inhibit the debug output
if bDebug = .true then
do
   say "--- CurrencyConverterController.rex (" pp(fn) "), at begin of prolog code" pp(d1)
   say "// CurrencyConverterController.rex, dumping SlotDir:"
   call dumpSlotDir arg(1)
   say
   say "// CurrencyConverterController.rex, dumping SlotDir~scriptContext:"
   call dumpScriptContext arg(1)~scriptContext
   say
end

call bsf.loadClass "java.text.NumberFormat", "NumberFormat"
call bsf.loadClass "java.math.RoundingMode", "RoundingMode"
call bsf.loadClass "java.lang.Math",         "Math"
call bsf.loadClass "javafx.scene.input.KeyEvent", "KeyEvent"

-- BSF.importClass allows ooRexx "new" messages to be sent to the imported class to create Java instances
call bsf.importClass "javafx.scene.control.Tooltip"        , "Tooltip"
call bsf.importClass "javafx.scene.control.Alert"          , "fxAlert"
call bsf.importClass "javafx.scene.control.Alert$AlertType", "fxAlertType"

-- Needed for beeping in a platform-independent form
.context~package~local~toolkit = bsf.loadClass("java.awt.Toolkit")~getDefaultToolkit

-- Store a properly configured currency number formatter in the package's local directory
pkgDir = .context~package~local    -- Get package's local environment directory
pkgDir~currencyF = .NumberFormat~getCurrencyInstance ~~setRoundingMode(.RoundingMode~half_up)
pkgDir~decimalF = bsf.loadClass('java.text.DecimalFormat')~getInstance

/* Use RexxScriptEngine facility "Rexx script annotation" to fetch ScriptContext supplied objects */
/* @get( idinputField idoutputField ) */
jrxObj = bsfCreateRexxProxy(.filterEventHandler~new,,"javafx.event.EventHandler")
idinputField~addEventFilter(.keyEvent~any, jrxObj)   -- Let us act on the events directed at any TextField component

/* Format the input field and output field with the decimal formatter */
call setDecimalPattern idinputField
call setDecimalPattern idoutputField

d2 = .dateTime~new
say "--- CurrencyConverterController.rex (" pp(fn) "), at end of prolog code" pp(d2) "duration:" pp(d2-d1)
say

::requires "BSF.CLS"

/* ------------------------------------------------------------------------- */
::routine setDecimalPattern   -- Determine type and set decimal formatting pattern accordingly
  use arg fxComponent
  id = fxComponent~getId        -- Correct way to retrieve ID in JavaFX
  if id~pos("idinputField") > 0 | id~pos("idoutputField") > 0 then .decimalF~applyPattern("#,##0.00")  -- Format to two decimal places
  else .decimalF~applyPattern("#,##0")   -- Format to not show any decimal places

/* ------------------------------------------------------------------------- */
::routine dumpScriptContext   -- Dump all current Bindings of ScriptContext
  use arg scriptContext
  if scriptContext~isA(.slot.argument) then scriptContext = scriptContext~scriptContext

  names = .directory~of( (100,"ENGINE"), (200,"GLOBAL") )  -- Standardized scope names
  do scope over scriptContext~getScopes
     if names~hasEntry(scope) then info = "(" names~entry(scope) ")"
                              else info = ""
     say "scope #" pp(scope) info
     bindings = scriptContext~getBindings(scope)
     dir = .directory~new
     len = 0
     do with index idx item it over bindings
        dir~setEntry(idx, (idx, it))
        len = max(len, idx~length)  -- Determine longest entry
     end
     len += 2                      -- Account for enclosing brackets
     do idx over dir~allIndexes~sort
        val = dir[idx]
        say "   " pp(idx)~left(len, ".") || ":" pp(val[1])~left(len, ".") "->" pp(val[2])
     end
     say
  end

/* ------------------------------------------------------------------------- */
::routine dumpSlotDir            -- Dump content of slotDir
  use arg slotDir
  do idx over slotDir~allIndexes~sort
      say "   " pp(idx)~left(31, ".") ":" pp(slotDir[idx])
  end

/* ------------------------------------------------------------------------- */
/* Will be invoked via JavaFX using the Java scripting framework */
::routine buttonClicked public
  /* @get( idinputField idoutputField idconvertButton idcurrency1Menu idcurrency2Menu ) */
  inputText = idinputField~text
  -- Perform currency conversion logic here
  outputText = inputText  -- Placeholder logic

  idoutputField~text = .currencyF~format(outputText)

/* ------------------------------------------------------------------------- */
/* Will be invoked directly via Rexx (no RexxScript annotations available) */
::class filterEventHandler public
::method handle   -- Method that controls numeric input within range in all three TextField's
  use arg event, slotDir

  target = event~target
  id = target~id                   -- Get "id" attribute value, can be .nil
  if id = .nil then return         -- Not interested in nodes that have no "id" set
  if id~pos("idinputField") = 0 then return -- Interested in key presses in TextField nodes only

  strEventType = event~eventType~toString

  if strEventType = "KEY_PRESSED" then   -- Save TextField's object, text and caret position
  do
     pkgDir = .context~package~local
     pkgDir~key_typed_target = target
     pkgDir~key_typed_text = target~text
     pkgDir~key_typed_caretPosition = target~caretPosition
     return
  end

  if strEventType = "KEY_TYPED" then
  do
     return
  end

  else if strEventType = "KEY_RELEASED" then
  do
     tfValue = target~text               -- Get the current text, should be a number
     pkgDir = .context~package~local     -- Get access to package's local environment (since ooRexx 5.0)
     if target~objectName = pkgDir~key_typed_target~objectName then   -- Same object as previous "KEY_TYPED" target
     do
         title = "" -- If set, an error title was defined
         if \datatype(tfValue, "Number") then   -- Check for a valid number
         do
             title = "Not a Number (NaN)"
             headerText = "The currently entered value" pp(tfValue) "is not a valid number!"
             contentText = "Resetting to previous value:" pp(pkgDir~key_typed_text)
         end
         else    -- Checks are alright, now update the slider marker
         do
            target~text = tfValue   -- Reflect new value in legible form
         end

              -- Error condition?
         if title <> "" then   -- Error information available, display it
         do
            .toolkit~beep    -- Beep to alert
             alert = .fxAlert~new(.fxAlertType~ERROR)          -- Create Alert dialog
             alert~~setTitle(title) ~~setHeaderText(headerText) ~~setContentText(contentText)
             res = alert~showAndWait      -- Show the dialog, wait until user closes it, returns a java.util.Optional (isPresent(), get(), ...)
             target~text = pkgDir~key_typed_text
             target~positionCaret(pkgDir~key_typed_caretPosition)
             return
         end
     end
  end
  return

/* ------------------------------------------------------------------------- */
/* JavaFX components and their event handlers */
/* Will be invoked directly via Rexx (no RexxScript annotations available) */
::method idconvertButtonAction
  call buttonClicked .context~slotDir


